package com.ruoyi.web.controller.enterprise;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.EnterprisePhoto;
import com.ruoyi.system.service.IEnterprisePhotoService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 企业文件信息Controller
 * 
 * @author ruoyi
 * @date 2023-02-20
 */
@RestController
@RequestMapping("/system/photo")
public class EnterprisePhotoController extends BaseController
{
    @Autowired
    private IEnterprisePhotoService enterprisePhotoService;

    /**
     * 查询企业文件信息列表
     */
    @PreAuthorize("@ss.hasPermi('system:photo:list')")
    @GetMapping("/list")
    public TableDataInfo list(EnterprisePhoto enterprisePhoto)
    {
        startPage();
        List<EnterprisePhoto> list = enterprisePhotoService.selectEnterprisePhotoList(enterprisePhoto);
        return getDataTable(list);
    }

    /**
     * 导出企业文件信息列表
     */
    @PreAuthorize("@ss.hasPermi('system:photo:export')")
    @Log(title = "企业文件信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EnterprisePhoto enterprisePhoto)
    {
        List<EnterprisePhoto> list = enterprisePhotoService.selectEnterprisePhotoList(enterprisePhoto);
        ExcelUtil<EnterprisePhoto> util = new ExcelUtil<EnterprisePhoto>(EnterprisePhoto.class);
        util.exportExcel(response, list, "企业文件信息数据");
    }

    /**
     * 获取企业文件信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:photo:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(enterprisePhotoService.selectEnterprisePhotoById(id));
    }

    /**
     * 新增企业文件信息
     */
    @PreAuthorize("@ss.hasPermi('system:photo:add')")
    @Log(title = "企业文件信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EnterprisePhoto enterprisePhoto)
    {
        return toAjax(enterprisePhotoService.insertEnterprisePhoto(enterprisePhoto));
    }

    /**
     * 修改企业文件信息
     */
    @PreAuthorize("@ss.hasPermi('system:photo:edit')")
    @Log(title = "企业文件信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EnterprisePhoto enterprisePhoto)
    {
        return toAjax(enterprisePhotoService.updateEnterprisePhoto(enterprisePhoto));
    }

    /**
     * 删除企业文件信息
     */
    @PreAuthorize("@ss.hasPermi('system:photo:remove')")
    @Log(title = "企业文件信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(enterprisePhotoService.deleteEnterprisePhotoByIds(ids));
    }
}
