package com.ruoyi.web.controller.kaoqin;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.system.domain.KaoqinConfig;
import com.ruoyi.system.service.IKaoqinConfigService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.KaoqinRecord;
import com.ruoyi.system.service.IKaoqinRecordService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 考勤记录Controller
 * 
 * @author ruoyi
 * @date 2023-02-02
 */
@RestController
@RequestMapping("/record")
public class KaoqinRecordController extends BaseController
{
    @Autowired
    private IKaoqinRecordService kaoqinRecordService;

    @Autowired
    private IKaoqinConfigService kaoqinConfigService;


    /**
     * 查询考勤记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:record:list')")
    @GetMapping("/list")
    public TableDataInfo list(KaoqinRecord kaoqinRecord)
    {
        startPage();
        List<KaoqinRecord> list = kaoqinRecordService.selectKaoqinRecordList(kaoqinRecord);
        KaoqinConfig kaoqinConfigDept = kaoqinConfigService.selectKaoqinConfigBynew();
        return getDataTableNum(list,kaoqinConfigDept.getKaNum());
    }

    /**
     * 导出考勤记录列表
     */
    @PreAuthorize("@ss.hasPermi('system:record:export')")
    @Log(title = "考勤记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, KaoqinRecord kaoqinRecord)
    {
        List<KaoqinRecord> list = kaoqinRecordService.selectKaoqinRecordList(kaoqinRecord);
        ExcelUtil<KaoqinRecord> util = new ExcelUtil<KaoqinRecord>(KaoqinRecord.class);
        util.exportExcel(response, list, "考勤记录数据");
    }

    /**
     * 获取考勤记录详细信息
     */

    @GetMapping(value = "/{recordId}")
    public AjaxResult getInfo(@PathVariable("recordId") Long recordId)
    {
        return AjaxResult.success(kaoqinRecordService.selectKaoqinRecordByRecordId(recordId));
    }

    /**
     * 新增考勤记录
     */
    @PreAuthorize("@ss.hasPermi('system:record:add')")
    @Log(title = "考勤记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody KaoqinRecord kaoqinRecord)
    {
        return toAjax(kaoqinRecordService.insertKaoqinRecord(kaoqinRecord));
    }

    /**
     * 修改考勤记录
     */
    @PreAuthorize("@ss.hasPermi('system:record:edit')")
    @Log(title = "考勤记录", businessType = BusinessType.UPDATE)
    @PostMapping("/put")
    public AjaxResult edit(@RequestBody KaoqinRecord kaoqinRecord)
    {
        return toAjax(kaoqinRecordService.updateKaoqinRecord(kaoqinRecord));
    }

    /**
     * 删除考勤记录
     */
    @PreAuthorize("@ss.hasPermi('system:record:remove')")
    @Log(title = "考勤记录", businessType = BusinessType.DELETE)
    @GetMapping(value = "/delete/{recordIds}")
    public AjaxResult remove(@PathVariable Long[] recordIds)
    {
        return toAjax(kaoqinRecordService.deleteKaoqinRecordByRecordIds(recordIds));
    }
}
