package com.ruoyi.web.controller.qinjia;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.RecordLeaveFj;
import com.ruoyi.system.service.IRecordLeaveFjService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 请假_附件Controller
 * 
 * @author ruoyi
 * @date 2025-06-19
 */
@RestController
@RequestMapping("/system/fj")
public class RecordLeaveFjController extends BaseController
{
    @Autowired
    private IRecordLeaveFjService recordLeaveFjService;

    /**
     * 查询请假_附件列表
     */
    @PreAuthorize("@ss.hasPermi('system:fj:list')")
    @GetMapping("/list")
    public TableDataInfo list(RecordLeaveFj recordLeaveFj)
    {
        startPage();
        List<RecordLeaveFj> list = recordLeaveFjService.selectRecordLeaveFjList(recordLeaveFj);
        return getDataTable(list);
    }

    /**
     * 导出请假_附件列表
     */
    @PreAuthorize("@ss.hasPermi('system:fj:export')")
    @Log(title = "请假_附件", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RecordLeaveFj recordLeaveFj)
    {
        List<RecordLeaveFj> list = recordLeaveFjService.selectRecordLeaveFjList(recordLeaveFj);
        ExcelUtil<RecordLeaveFj> util = new ExcelUtil<RecordLeaveFj>(RecordLeaveFj.class);
        util.exportExcel(response, list, "请假_附件数据");
    }

    /**
     * 获取请假_附件详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:fj:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(recordLeaveFjService.selectRecordLeaveFjById(id));
    }

    /**
     * 新增请假_附件
     */
    @PreAuthorize("@ss.hasPermi('system:fj:add')")
    @Log(title = "请假_附件", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RecordLeaveFj recordLeaveFj)
    {
        return toAjax(recordLeaveFjService.insertRecordLeaveFj(recordLeaveFj));
    }

    /**
     * 修改请假_附件
     */
    @PreAuthorize("@ss.hasPermi('system:fj:edit')")
    @Log(title = "请假_附件", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RecordLeaveFj recordLeaveFj)
    {
        return toAjax(recordLeaveFjService.updateRecordLeaveFj(recordLeaveFj));
    }

    /**
     * 删除请假_附件
     */
    @PreAuthorize("@ss.hasPermi('system:fj:remove')")
    @Log(title = "请假_附件", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(recordLeaveFjService.deleteRecordLeaveFjByIds(ids));
    }
}
