package com.ruoyi.system.service.impl;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.domain.BusinessUser;
import com.ruoyi.system.domain.ProcessConfiguration;
import com.ruoyi.system.mapper.BusinessUserMapper;
import com.ruoyi.system.mapper.ProcessConfigurationMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.BusinessTripMapper;
import com.ruoyi.system.domain.BusinessTrip;
import com.ruoyi.system.service.IBusinessTripService;

/**
 * 出差记录信息Service业务层处理
 * 
 * @author ruoyi
 * @date 2025-06-23
 */
@Service
public class BusinessTripServiceImpl implements IBusinessTripService 
{
    @Autowired
    private BusinessTripMapper businessTripMapper;

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private BusinessUserMapper businessUserMapper;

    @Autowired
    private ProcessConfigurationMapper processConfigurationMapper;

    /**
     * 查询出差记录信息
     * 
     * @param id 出差记录信息主键
     * @return 出差记录信息
     */
    @Override
    public BusinessTrip selectBusinessTripById(Long id)
    {
        BusinessTrip businessTrip = businessTripMapper.selectBusinessTripById(id);

        BusinessUser businessUser = new BusinessUser();
        businessUser.setBusinessId(id);
        List<BusinessUser> businessUsers = businessUserMapper.selectBusinessUserList(businessUser);
        businessTrip.setBusinessUserList(businessUsers);

        return businessTrip;
    }

    /**
     * 查询出差记录信息列表
     * 
     * @param businessTrip 出差记录信息
     * @return 出差记录信息
     */
    @Override
    public List<BusinessTrip> selectBusinessTripList(BusinessTrip businessTrip)
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        if (!"admin".equals(user.getUserName())){
            businessTrip.setUserId(user.getUserId());
        }
        return businessTripMapper.selectBusinessTripListLb(businessTrip);
    }

    /**
     * 新增出差记录信息
     * 
     * @param businessTrip 出差记录信息
     * @return 结果
     */
    @Override
    public int insertBusinessTrip(BusinessTrip businessTrip)
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        businessTrip.setInitiatorId(user.getUserId());
        businessTrip.setInitiatorName(user.getNickName());
        businessTrip.setSubmitTime(DateUtils.getNowDate());
        int index = businessTripMapper.insertBusinessTrip(businessTrip);

        BusinessUser bu = new BusinessUser();
        bu.setAbsenteeId(user.getUserId());
        bu.setAbsenteeName(user.getNickName());
        bu.setDeptId(user.getDeptId());
        bu.setDeptName(user.getDept().getDeptName());
        bu.setType("1");
        bu.setSubmitTime(DateUtils.getNowDate());
        businessUserMapper.insertBusinessUser(bu);

        if(businessTrip.getBusinessUserList()!=null && !businessTrip.getBusinessUserList().isEmpty()){
            //保存人员表
            StringBuilder sb = new StringBuilder();
            List<BusinessUser> businessUserList = businessTrip.getBusinessUserList();
            for (int i = 0; i < businessUserList.size(); i++) {
                BusinessUser businessUser = businessUserList.get(i);

                if(i==0){
                    sb.append(businessUser.getAbsenteeName());
                }else{
                    sb.append("、").append(businessUser.getAbsenteeName());
                }


                businessUser.setBusinessId(businessTrip.getId());
                //查询人员部门信息
                SysUser sysUser = userMapper.selectUserById(businessUser.getAbsenteeId());
                businessUser.setDeptId(sysUser.getDeptId());
                businessUser.setDeptName(sysUser.getDept().getDeptName());
                businessUser.setType("2");
                businessUser.setSubmitTime(DateUtils.getNowDate());
                businessUserMapper.insertBusinessUser(businessUser);
            }
            //将随行人员放入主表
            businessTrip.setAbsenteeName(sb.toString());
            businessTripMapper.updateBusinessTrip(businessTrip);
        }

        return index;
    }

    /**
     * 修改出差记录信息
     * 
     * @param businessTrip 出差记录信息
     * @return 结果
     */
    @Override
    public int updateBusinessTrip(BusinessTrip businessTrip)
    {
        if(!"0".equals(businessTrip.getIsPass())){
            throw new ServiceException("当前出差已处理，无法修改");
        }
        //删除随行人员信息
        businessUserMapper.deleteBusinessUserByBusinessId(businessTrip.getId());

        SysUser use = SecurityUtils.getLoginUser().getUser();
        BusinessUser bUse = new BusinessUser();
        bUse.setAbsenteeId(use.getUserId());
        bUse.setAbsenteeName(use.getNickName());
        bUse.setDeptId(use.getDeptId());
        bUse.setDeptName(use.getDept().getDeptName());
        bUse.setType("1");
        bUse.setSubmitTime(DateUtils.getNowDate());
        businessUserMapper.insertBusinessUser(bUse);

        if(businessTrip.getBusinessUserList()!=null && !businessTrip.getBusinessUserList().isEmpty()){
            //保存人员表
            StringBuilder sb = new StringBuilder();
            List<BusinessUser> businessUserList = businessTrip.getBusinessUserList();
            for (int i = 0; i < businessUserList.size(); i++) {
                BusinessUser businessUser = businessUserList.get(i);

                if(i==0){
                    sb.append(businessUser.getAbsenteeName());
                }else{
                    sb.append("、").append(businessUser.getAbsenteeName());
                }

                businessUser.setBusinessId(businessTrip.getId());
                //查询人员部门信息
                SysUser sysUser = userMapper.selectUserById(businessUser.getAbsenteeId());
                businessUser.setDeptId(sysUser.getDeptId());
                businessUser.setDeptName(sysUser.getDept().getDeptName());
                businessUser.setType("2");
                businessUser.setSubmitTime(DateUtils.getNowDate());
                businessUserMapper.insertBusinessUser(businessUser);
            }


            //将随行人员放入主表
            businessTrip.setAbsenteeName(sb.toString());
        }

        return businessTripMapper.updateBusinessTrip(businessTrip);
    }

    /**
     * 批量删除出差记录信息
     * 
     * @param ids 需要删除的出差记录信息主键
     * @return 结果
     */
    @Override
    public int deleteBusinessTripByIds(Long[] ids)
    {
        return businessTripMapper.deleteBusinessTripByIds(ids);
    }

    /**
     * 删除出差记录信息信息
     * 
     * @param id 出差记录信息主键
     * @return 结果
     */
    @Override
    public int deleteBusinessTripById(Long id)
    {
        return businessTripMapper.deleteBusinessTripById(id);
    }

    @Override
    public int audit(BusinessTrip businessTrip) {
        //获取当前人员信息
        SysUser user = SecurityUtils.getLoginUser().getUser();
        businessTrip.setExaminersId(user.getUserId());
        businessTrip.setExaminersName(user.getNickName());
        return businessTripMapper.updateBusinessTrip(businessTrip);
    }

    @Override
    public List<BusinessTrip> presentList(BusinessTrip businessTrip) {

        //获取当前人员信息
        SysUser user = SecurityUtils.getLoginUser().getUser();
        if (!"admin".equals(user.getUserName())){
            //获取当前人员补卡配置所在的部门id
            ProcessConfiguration processConfiguration = new ProcessConfiguration();
            processConfiguration.setUserId(user.getUserId());
            processConfiguration.setType("3");
            List<ProcessConfiguration> list = processConfigurationMapper.selectProcessConfigurationList(processConfiguration);
            List<Long> deptIds = list.stream().map(ProcessConfiguration::getDeptId).collect(Collectors.toList());
            businessTrip.setDeptIds(deptIds);
        }
        return businessTripMapper.selectBusinessTripList(businessTrip);
    }
}
