package com.ruoyi.system.service.impl;

import java.util.List;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.domain.KaoqinRecord;
import com.ruoyi.system.mapper.KaoqinRecordMapper;
import com.ruoyi.system.service.IKaoqinRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


/**
 * 考勤记录Service业务层处理
 * 
 * @author boman
 * @date 2023-02-09
 */
@Service
public class KaoqinRecordServiceImpl implements IKaoqinRecordService
{
    @Autowired
    private KaoqinRecordMapper kaoqinRecordMapper;

    /**
     * 查询考勤记录
     * 
     * @param recordId 考勤记录主键
     * @return 考勤记录
     */
    @Override
    public KaoqinRecord selectKaoqinRecordByRecordId(Long recordId)
    {
        return kaoqinRecordMapper.selectKaoqinRecordByRecordId(recordId);
    }

    /**
     * 查询考勤记录列表
     * 
     * @param kaoqinRecord 考勤记录
     * @return 考勤记录
     */
    @Override
    public List<KaoqinRecord> selectKaoqinRecordList(KaoqinRecord kaoqinRecord)
    {
        return kaoqinRecordMapper.selectKaoqinRecordList(kaoqinRecord);
    }

    /**
     * 新增考勤记录
     * 
     * @param kaoqinRecord 考勤记录
     * @return 结果
     */
    @Override
    public int insertKaoqinRecord(KaoqinRecord kaoqinRecord)
    {
        //获取当前人员信息
        SysUser user = SecurityUtils.getLoginUser().getUser();
        String ancestors = kaoqinRecord.getAncestors();
        Long deptId = kaoqinRecord.getDeptId();
        ancestors = ancestors +","+ deptId;
        kaoqinRecord.setAncestors(ancestors);
        kaoqinRecord.setCreateTime(DateUtils.getNowDate());
        kaoqinRecord.setPhone(user.getPhonenumber());
        return kaoqinRecordMapper.insertKaoqinRecord(kaoqinRecord);
    }

    /**
     * 修改考勤记录
     * 
     * @param kaoqinRecord 考勤记录
     * @return 结果
     */
    @Override
    public int updateKaoqinRecord(KaoqinRecord kaoqinRecord)
    {
        kaoqinRecord.setUpdateTime(DateUtils.getNowDate());
        return kaoqinRecordMapper.updateKaoqinRecord(kaoqinRecord);
    }

    /**
     * 批量删除考勤记录
     * 
     * @param recordIds 需要删除的考勤记录主键
     * @return 结果
     */
    @Override
    public int deleteKaoqinRecordByRecordIds(Long[] recordIds)
    {
        return kaoqinRecordMapper.deleteKaoqinRecordByRecordIds(recordIds);
    }

    /**
     * 删除考勤记录信息
     * 
     * @param recordId 考勤记录主键
     * @return 结果
     */
    @Override
    public int deleteKaoqinRecordByRecordId(Long recordId)
    {
        return kaoqinRecordMapper.deleteKaoqinRecordByRecordId(recordId);
    }
}
