package com.ruoyi.system.service.impl;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.domain.CardReplacementRecord;
import com.ruoyi.system.domain.ProcessConfiguration;
import com.ruoyi.system.domain.RecordLeaveFj;
import com.ruoyi.system.mapper.ProcessConfigurationMapper;
import com.ruoyi.system.mapper.RecordLeaveFjMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.RecordLeaveMapper;
import com.ruoyi.system.domain.RecordLeave;
import com.ruoyi.system.service.IRecordLeaveService;

/**
 * 请假记录信息Service业务层处理
 * 
 * @author ruoyi
 * @date 2023-02-13
 */
@Service
public class RecordLeaveServiceImpl implements IRecordLeaveService 
{
    @Autowired
    private RecordLeaveMapper recordLeaveMapper;

    @Autowired
    private ProcessConfigurationMapper processConfigurationMapper;

    @Autowired
    private RecordLeaveFjMapper recordLeaveFjMapper;

    /**
     * 查询请假记录信息
     * 
     * @param id 请假记录信息主键
     * @return 请假记录信息
     */
    @Override
    public RecordLeave selectRecordLeaveById(Long id)
    {
        return recordLeaveMapper.selectRecordLeaveById(id);
    }

    /**
     * 查询请假记录信息列表
     * 
     * @param recordLeave 请假记录信息
     * @return 请假记录信息
     */
    @Override
    public List<RecordLeave> selectRecordLeaveList(RecordLeave recordLeave)
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        recordLeave.setAbsenteeId(String.valueOf(user.getUserId()));
        return recordLeaveMapper.selectRecordLeaveList(recordLeave);
    }

    /**
     * 新增请假记录信息
     * 
     * @param recordLeave 请假记录信息
     * @return 结果
     */
    @Override
    public int insertRecordLeave(RecordLeave recordLeave)
    {
        //获取当前人员信息
        SysUser user = SecurityUtils.getLoginUser().getUser();
        recordLeave.setAbsenteeId(String.valueOf(user.getUserId()));
        recordLeave.setAbsenteeName(user.getNickName());
        recordLeave.setDeptId(user.getDeptId());
        recordLeave.setDeptName(user.getDept().getDeptName());
        recordLeave.setSubmitTime(DateUtils.getNowDate());

        return recordLeaveMapper.insertRecordLeave(recordLeave);
    }

    /**
     * 修改请假记录信息
     * 
     * @param recordLeave 请假记录信息
     * @return 结果
     */
    @Override
    public int updateRecordLeave(RecordLeave recordLeave)
    {
        RecordLeave recordLeave1 = recordLeaveMapper.selectRecordLeaveById(recordLeave.getId());
        if(!"0".equals(recordLeave1.getIsPass())){
            throw new ServiceException("当前补卡已处理，无法修改");
        }
        recordLeave.setSubmitTime(DateUtils.getNowDate());
        return recordLeaveMapper.updateRecordLeave(recordLeave);
    }

    /**
     * 批量删除请假记录信息
     * 
     * @param ids 需要删除的请假记录信息主键
     * @return 结果
     */
    @Override
    public int deleteRecordLeaveByIds(Long[] ids)
    {
        return recordLeaveMapper.deleteRecordLeaveByIds(ids);
    }

    /**
     * 删除请假记录信息信息
     * 
     * @param id 请假记录信息主键
     * @return 结果
     */
    @Override
    public int deleteRecordLeaveById(Long id)
    {
        return recordLeaveMapper.deleteRecordLeaveById(id);
    }

    @Override
    public int audit(RecordLeave recordLeave) {
        //获取当前人员信息
        SysUser user = SecurityUtils.getLoginUser().getUser();
        recordLeave.setExaminersId(user.getUserId());
        recordLeave.setExaminersName(user.getNickName());
        return recordLeaveMapper.updateRecordLeave(recordLeave);
    }

    @Override
    public List<RecordLeave> presentList(RecordLeave recordLeave) {
        //获取当前人员信息
        SysUser user = SecurityUtils.getLoginUser().getUser();
        if (!"admin".equals(user.getUserName())){
            //获取当前人员补卡配置所在的部门id
            ProcessConfiguration processConfiguration = new ProcessConfiguration();
            processConfiguration.setUserId(user.getUserId());
            processConfiguration.setType("2");
            List<ProcessConfiguration> list = processConfigurationMapper.selectProcessConfigurationList(processConfiguration);
            List<Long> deptIds = list.stream().map(ProcessConfiguration::getDeptId).collect(Collectors.toList());
            recordLeave.setDeptIds(deptIds);
        }
        return recordLeaveMapper.selectRecordLeaveList(recordLeave);
    }
}
